/*
//
// BEGIN SONGBIRD GPL
//
// This file is part of the Songbird web player.
//
// Copyright(c) 2005-2008 POTI, Inc.
// http://songbirdnest.com
//
// The Initial Developer of the Original Code is
// Steven Bengtson.
// Portions created by the Initial Developer are Copyright (C) 2008
// the Initial Developer. All Rights Reserved.
//
// Contributor(s):
//
// This file may be licensed under the terms of of the
// GNU General Public License Version 2 (the "GPL").
//
// Software distributed under the License is distributed
// on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either
// express or implied. See the GPL for the specific language
// governing rights and limitations.
//
// You should have received a copy of the GPL along with this
// program. If not, go to http://www.gnu.org/licenses/gpl.html
// or write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
//
// END SONGBIRD GPL
//
*/

// Constants for convience
const Cc = Components.classes;
const Ci = Components.interfaces;

Components.utils.import("resource://app/components/sbProperties.jsm");
Components.utils.import("resource://app/jsmodules/sbLibraryUtils.jsm");

var AlbumArtManager = {
  _AlbumArtBox: null,       // Container for all the albums
  _AlbumStatusLabel: null,  // Label on the status bar
  _AlbumStatusImage: null,  // Image (spinner) on the status bar

  _albumArtService: null,   // Album Art Service
  _bundle: null,            // String bundle
  _preferences: null,       // Preferences Service

  DEBUG: false,              // Set to true for debug output on console

  _debug: function (message)
  {
    if (!this.DEBUG) return;

    var messageArray = message.split("\n");
    for (msg in messageArray)
    {
      dump("AlbumArtManager: " + messageArray[msg] + "\n");
    }
  },

  _logError: function (message)
  {
    var messageArray = message.split("\n");
    for (msg in messageArray)
    {
      dump("AlbumArtManager: [ERROR] - " + messageArray[msg] + "\n");
    }
  },

  /**
   * sbIAlbumArtListener
   */
  onAlbumArtChange: function(aMediaItem) {
    this._debug("onAlbumArtChange checking aMediaItem");
    this._debug("onAlbumArtChange: " + aMediaItem);

    var mAlbum = aMediaItem.getProperty(SBProperties.albumName);
    var aAlbumElement = this._findExistingAlbum(mAlbum);

    if (aAlbumElement) {
      this._toggleSpinner(false, aAlbumElement);
      var albumCover = aMediaItem.getProperty(SBProperties.primaryImageURL);
      aAlbumElement.setAttribute("src", albumCover);
      this._setStatusLabel("albumartmanager.status.searchfound", [mAlbum]);
    }
  },

  onAlbumArtChangeError: function(aMediaItem) {
    this._debug("onAlbumArtChangeError checking aMediaItem");
    this._debug("onAlbumArtChangeError: " + aMediaItem);
    var mAlbum = aMediaItem.getProperty(SBProperties.albumName);
    var aAlbumElement = this._findExistingAlbum(mAlbum);

    if (aAlbumElement) {
      this._toggleSpinner(false, aAlbumElement);
      this._setStatusLabel("albumartmanager.status.searchfailed", [mAlbum]);
    }
  },

  /**
   * AlbumArtManager functions
   */
  init: function() {
    this._mainLibrary = LibraryUtils.mainLibrary;

    this._preferences = Cc['@mozilla.org/preferences-service;1']
                        .getService(Ci.nsIPrefBranch);
    this._sbVersion = this._preferences
                          .getCharPref("extensions.lastAppVersion");

    this._AlbumArtBox = document.getElementById("AlbumArtManager_AlbumList");
    this._AlbumStatusLabel = document.getElementById("AlbumArtManager_StatusLabel");
    this._AlbumStatusImage = document.getElementById("AlbumArtManager_StatusImage");

    var stringBundleService = Cc["@mozilla.org/intl/stringbundle;1"]
                              .getService(Ci.nsIStringBundleService);
    this._bundle = stringBundleService
    .createBundle("chrome://albumartmanager/locale/albumartmanager.properties");

    this._albumArtService = Cc["@songbirdnest.com/songbird-album-art-service;1"]
                            .getService(Ci.sbIAlbumArtService);
    this._albumArtService.addListener(this);

    this._getAlbums();
  },

  openPrefs: function() {
    if (window && window.SBOpenPreferences) {
      window.SBOpenPreferences("library_albumartmanager_pref_pane");
    } else {
      var windowMediator;
      var sbWindow;

      /* Get the main Songbird window. */
      windowMediator = Components.classes
                          ["@mozilla.org/appshell/window-mediator;1"].
                          getService(Components.interfaces.nsIWindowMediator);
      sbWindow = windowMediator.getMostRecentWindow("Songbird:Main");

      /* Open the preferences pane. */
      sbWindow.SBOpenPreferences("library_albumartmanager_pref_pane");
    }
  },

  closeWindow: function() {
    window.close();
  },

  clearAlbumArt: function(aMediaItemGuid, aAlbumElement) {
    this._debug("Calling clearAlbumCover on " + aMediaItem);
    var mMediaItem = this._mainLibrary.getMediaItem(aMediaItemGuid);

    if (!mMediaItem) {
      return;
    }

    if (aAlbumElement) {
      this._toggleSpinner(true, aAlbumElement);
    }

    this._albumArtService.clearAlbumCover(mMediaItem);
  },

  getAlbumArt: function(aMediaItemGuid, aAlbumElement, service) {
    var mMediaItem = this._mainLibrary.getMediaItem(aMediaItemGuid);

    if (!mMediaItem) {
      return true;
    }

    var mAlbum = mMediaItem.getProperty(SBProperties.albumName);
    if (aAlbumElement) {
      this._toggleSpinner(true, aAlbumElement);
      this._setStatusLabel("albumartmanager.status.searchingforCover", [mAlbum]);
    }

    var coverGetter = null;
    switch (service) {
      case 'Amazon':
        this._albumArtService.getAlbumCoverFromAmazon(mMediaItem, false, false);
      break;
      case 'File':
      default:
        // Have to manually hide the menu or it pops up after the file picker
        // very odd.
        var albumMenu = document.getAnonymousElementByAttribute(albumElement,
                                                                  "sid",
                                                                  "albumMenu");
        albumMenu.hidePopup();
        this._albumArtService.getAlbumCoverFromFile(window, mMediaItem);
      break;
    }

    return true;
  },

  _findExistingAlbum: function(albumName) {
    if (this._AlbumArtBox.hasChildNodes()) {
      var currentNode = this._AlbumArtBox.firstChild;
      while (currentNode) {
        var mAlbumName = currentNode.getAttribute("value");
        this._debug("Checking " + mAlbumName + " against " + albumName);
        if ( mAlbumName == albumName ) {
          return currentNode;
        }
        currentNode = currentNode.nextSibling;
      }
    }
    return null;
  },

  _toggleSpinner: function(aSpinnerOn, aAlbumElement) {
    if (aAlbumElement.toggleSpinner) {
      aAlbumElement.toggleSpinner(aSpinnerOn);
    }
  },

  _toggleStatusSpinner: function(aSpinnerOn) {
    if (aSpinnerOn) {
      this._AlbumStatusImage.removeAttribute("hidden");
    } else {
      this._AlbumStatusImage.setAttribute("hidden", "true");
    }
  },

  /**
   * Album Scanning
   */
  _addAlbum: function(aMediaItem) {
    var mAlbum = aMediaItem.getProperty(SBProperties.albumName);
    if (mAlbum == null || mAlbum == "") {
      return;
    }

    var mLocation = aMediaItem.getProperty(SBProperties.contentURL);
    if (mLocation.indexOf("file://") != 0) {
      return;
    }

    this._debug("Adding album: " + mAlbum);
    var mAlbumArtUrl = aMediaItem.getProperty(SBProperties.primaryImageURL);

    if (mAlbumArtUrl == null || mAlbumArtUrl == "") {
      mAlbumArtUrl = this._albumArtService.defaultCover;
    }

    var newAlbumElement = document.createElement("albumplace");
    newAlbumElement.setAttribute("src", mAlbumArtUrl);
    newAlbumElement.setAttribute("value", mAlbum);
    newAlbumElement.setAttribute("mediaItemGuid", aMediaItem.guid);
    this._AlbumArtBox.appendChild(newAlbumElement);
  },

  _getAlbums: function() {
    var mainLocalLibrary = this._mainLibrary.QueryInterface(Ci.sbILocalDatabaseLibrary);
    this._array = Cc["@songbirdnest.com/Songbird/Library/LocalDatabase/AsyncGUIDArray;1"]
                   .createInstance(Ci.sbILocalDatabaseAsyncGUIDArray);
    this._array.databaseGUID = mainLocalLibrary.databaseGuid;
    this._array.propertyCache = mainLocalLibrary.propertyCache;
    this._array.baseTable = 'media_items';
    this._array.isDistinct = true;
    this._array.addSort(SBProperties.albumName, true);
    this._array.addAsyncListener(this);
    this._array.invalidate();
    this._array.getLengthAsync();
  },

  _setStatusLabel: function(aNewLabel, aExtraData) {
    var bundleString = aNewLabel;
    try {
      if (aExtraData) {
        bundleString = this._bundle.formatStringFromName(aNewLabel,
                                                         aExtraData,
                                                         aExtraData.length);
      } else {
        bundleString = this._bundle.GetStringFromName(aNewLabel);
      }
      this._AlbumStatusLabel.setAttribute("label", bundleString);
    } catch (err) {
      this._AlbumStatusLabel.setAttribute("label", err);
    }
  },


  /**
   * sbILocalDatabaseAsyncGUIArrayListener
   */
  onGetGuidByIndex: function(aIndex, aGuid, aResult) {
    this._debug("Found [" + aIndex + " of " + this._length + "] guid: " + aGuid);
    this._setStatusLabel("albumartmanager.status.searchingfor",
                         [aIndex, this._length]);

    if (aGuid) {
      var item = this._mainLibrary.getMediaItem(aGuid);
      var mAlbum = item.getProperty(SBProperties.albumName);
      this._setStatusLabel("albumartmanager.status.searchingforAlbum",
                           [aIndex, this._length, mAlbum]);
      this._addAlbum(item);
    }

    if (this._length > aIndex) {
      this._array.getGuidByIndexAsync(++aIndex);
    } else {
      this._setStatusLabel("albumartmanager.status.searchcomplete",
                           [this._length]);
      this._toggleStatusSpinner(false);
    }
  },

  onGetLength: function(aLength, aResult) {
    this._length = aLength;
    if (this._length > 0) {
      this._array.getGuidByIndexAsync(0);
    } else {
      this._setStatusLabel("albumartmanager.status.searchcomplete",
                           [this._length]);
      this._toggleStatusSpinner(false);
    }
  },

  QueryInterface: function(iid) {
    if (!iid.equals(Ci.sbILocalDatabaseAsyncGUIDArrayListener) &&
        !iid.equals(Ci.nsISupportsWeakReference) &&
        !iid.equals(Ci.nsISupports))
      throw Cr.NS_ERROR_NO_INTERFACE;
    return this;
  }


};
