// Originally developed by FoxyTunes modified by Steven Bengtson (steven@songbirdnest.com)
// for the Album Art Manager extension for Songbrid.

// Copyright 2006 FoxyTunes
// All Rights Reserved

Components.utils.import("resource://app/components/sbProperties.jsm");

/**
 * \brief Class: AmazonAlbumCoverGetter searches for and downloads album art
 *        from amazon.
 */
function AmazonAlbumCoverGetter(aMediaItem, location, handler) {
  this._album = aMediaItem.getProperty(SBProperties.albumName);;
  this._artist = aMediaItem.getProperty(SBProperties.artistName);
  this._location = location;
  this._handler = handler;
  this._requestObject = null;

  this._amazonKey = '0ACF688NVCRN8HN5FGG2';
  this._amazonTag = 'albumartmanager-sb';
  this._amazonURL = "http://webservices.amazon.%tld%/onca/xml?Service=AWSECommerceService&AWSAccessKeyId=%amazon_key%&AssociateTag=%amazon_tag%&Operation=ItemSearch&SearchIndex=Music%artist-query%%album-query%&ResponseGroup=Small,Images%additional-response-group%";
}

AmazonAlbumCoverGetter.prototype = {
  DEBUG: true,
  
  /**
   * \brief Dumps out a message if the DEBUG flag is enabled with
   *        "AmazonAlbumCoverGetter:" pre-appended.
   * \param message String to print out, the function will separate the string
   *                on (\n) new line characters.
   */
  _debug: function (message)
  {
    if (!this.DEBUG) return;
  
    var messageArray = message.split("\n");
    for (msg in messageArray)
    {
      dump("AmazonAlbumCoverGetter: " + messageArray[msg] + "\n");
    }
  },
  
  /**
   * \brief Dumps out an error message with "AmazonAlbumCoverGetter: [ERROR]"
   *        pre-appended.
   * \param message String to print out, the function will separate the string
   *                on (\n) new line characters.
   */
  _logError: function (message)
  {
    var messageArray = message.split("\n");
    for (msg in messageArray)
    {
      dump("AmazonAlbumCoverGetter: [ERROR] - " + messageArray[msg] + "\n");
    }    
  },

  /**
   * \brief Initalize the cover getter, just checks that we have valid
   *        information to query with.
   * \returns true if everything is ok, false if not.
   */
  init: function() {
    if (!this._album) {
      return false;
    }

    if (!this._location) {
      return false;
    }
    return true;
  },

  /**
   * \brief Runs the query and downloads the album art if found.
   */
  get: function() {
    var url = this._generateUrl('com');
    var me = this;

    this._requestObject = Components.classes["@mozilla.org/xmlextras/xmlhttprequest;1"]
                            .createInstance(Components.interfaces.nsIXMLHttpRequest);
    this._requestObject.open('get', url, true);
    this._requestObject.onreadystatechange = function() {
      if (me._requestObject.readyState == 4) {
        // for bug in mozilla that throw error when calling status
        var status =  -1;
        try {
          status = me._requestObject.status;
        } catch (e) {
        }
        
        if (status == 200) {
          // Check if not aborted
          if (Components.isSuccessCode(me._requestObject.channel.status)) {
            var albumCoverUrl = me._parseResponse();
            me._saveCover(albumCoverUrl)
            return;
          }
        } else {
          /* 200 response */
          me._handler.onError();
          return;
        }

        me._handler.onError();
      } /* ready state 4 */
    } /* onreadystatechange function */

    this._requestObject.send(null);
    return;
  },

  _getImageNameByType: function(imageType) {
    switch (imageType) {
      case 'image/jpeg': 
        imageExt = '.jpg';
      break;
        case 'image/gif': 
          imageExt = '.gif';
        break;
      case 'image/png': 
        imageExt = '.png';
      break;
      case 'image/bmp': 
        imageExt = '.bmp';
      break;
      case 'image/tiff': 
        imageExt = '.tiff';
      break;
      default:
        return null;
    }
    return imageExt;
  },

  _writeBinaryFile: function(file, bytes) {
    var fileOutputStream =  Cc["@mozilla.org/network/file-output-stream;1"]
                               .createInstance(Ci.nsIFileOutputStream);      
    var binStream = Cc["@mozilla.org/binaryoutputstream;1"]
                               .createInstance(Ci.nsIBinaryOutputStream);
    try {
      // Insure that the file and all parent folders are created.
      file.create(Ci.nsIFile.NORMAL_FILE_TYPE, 0777);
    } catch (e) {
      this._logError("Unable to create file: " + file.path + " - " + e);
      // maybe it already exists, try to remove it so we can replace it.
      try {
        file.remove(false);
      } catch (e) {
        this._logError("Unable to remove file: " + file.path + " - " + e);
        return false;
      }
    }

    try {
      // Now create the stream to write the bytes
      fileOutputStream.init(file, 0x02 | 0x08 | 0x20, 0777, 0);
      binStream.setOutputStream(fileOutputStream);
      binStream.writeBytes(bytes, bytes.length);
      binStream.close();
      fileOutputStream.close();
    } catch (e) {
      this._logError("Failed to write binary file: " + file.path + " - " + e);
      try { 
        binStream.close();
        fileOutputStream.close();
      } catch (e) {}
      return false;
    }
    return true;
  },

  _saveCover: function(aCoverUrl) {
    var inputStream = null; 
    var bytes = "";
    var ioService = Cc['@mozilla.org/network/io-service;1']
                      .getService(Ci.nsIIOService);
    var channel = ioService.newChannel(aCoverUrl, null, null)
                      .QueryInterface(Ci.nsIHttpChannel);
    var me = this;
    var listener = {
      onDataAvailable : function(request, context, aInputStream, offset, count) {
        inputStream.setInputStream(aInputStream);
        bytes += inputStream.readBytes(count);
      },

      onStartRequest : function(request, context) {
        inputStream = Cc['@mozilla.org/binaryinputstream;1'].
                        createInstance(Ci.nsIBinaryInputStream);
      },

      onStopRequest : function(request, context, statusCode) {
        var httpChannel = channel;
        var succeeded = null;
        try {
          succeeded = httpChannel.requestSucceeded;
        } catch (e) {}

        if (httpChannel && 
            succeeded && 
            Components.isSuccessCode(statusCode) &&
            (bytes.length > 0)) {
          var imageType = httpChannel.getResponseHeader('Content-Type');
          var file = Cc["@mozilla.org/file/local;1"].createInstance(Ci.nsILocalFile);
          var albumPath = me._location + me._getImageNameByType(imageType);

          file.initWithPath(albumPath);
          if (me._writeBinaryFile(file, bytes)) {
            albumPath = "file://" + file.path;
            me._handler.onSuccess(albumPath);
          } else {
            me._handler.onError();
          }
        }
      }
    }

    channel.asyncOpen(listener, null);
  },

  _parseResponse: function() {
    var albumCoverUrl = null;
    try {
      var doc = this._requestObject.responseXML;
      this._debug("Response from Amazon: [" + doc + "]");
      itemElements = doc.getElementsByTagName("Item");
      if (itemElements != null) {
        albumCoverUrl = this._findMostReleventItem(itemElements)
      }
    } catch (err) {
    }  
    return albumCoverUrl;
  },
  
  _getAlbumCoverDetailsBySize : function(itemElement, size) {
    var albumCoverUrl = null;

    var albumCoverSize = itemElement.getElementsByTagName(size + "Image")[0]
    if (albumCoverSize) {
      var albumCoverUrlNode = albumCoverSize.getElementsByTagName("URL")[0];
      if (albumCoverUrlNode) {
        albumCoverUrl = albumCoverUrlNode.firstChild.nodeValue;
      }
    }

    return albumCoverUrl;
  },

  _findMostReleventItem: function(itemElements) {
    var albumCoverUrl = null;
    
    for (var i=0; i < itemElements.length; i++) {
      var currentItemElement = itemElements[i];

      try {
        albumCoverUrl = this._getAlbumCoverDetailsBySize(currentItemElement, "Large");
        if (!albumCoverUrl) {
          albumCoverUrl = this._getAlbumCoverDetailsBySize(currentItemElement, "Medium");
          if (!albumCoverUrl) {
            albumCoverUrl = this._getAlbumCoverDetailsBySize(currentItemElement, "Small");
          }
        }
      } catch (err) {
        continue;
      }

      if (!albumCoverUrl) {
        continue;
      }

      break;
    }
    
    return albumCoverUrl;
  },

  _generateUrl: function(tld) {
    var url = this._amazonURL;
    
    if (this._artist != "") {
      var escapeArtist = escape(this._artist);
      url = this._amazonURL.replace(/%artist-query%/, "&Artist=" + escapeArtist);
    } else {
      url = this._amazonURL.replace(/%artist-query%/, "");
    }
      
    var responseGroup = "";
    if (this._album != "") {
      var escapeAlbum = escape(this._album);
      url = url.replace(/%album-query%/, "&Title=" + escapeAlbum);
    } else {
      url = url.replace(/%album-query%/, "");
      responseGroup = ",Tracks";
    }
    url = url.replace(/%additional-response-group%/, responseGroup);
    
    url = url.replace(/%tld%/, tld);
    url = url.replace(/%amazon_key%/, this._amazonKey);
    url = url.replace(/%amazon_tag%/, this._amazonTag);
    return url;
  }
}
