/*
//
// BEGIN SONGBIRD GPL
//
// This file is part of the Songbird web player.
//
// Copyright(c) 2005-2008 POTI, Inc.
// http://songbirdnest.com
//
// This file may be licensed under the terms of of the
// GNU General Public License Version 2 (the "GPL").
//
// Software distributed under the License is distributed
// on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either
// express or implied. See the GPL for the specific language
// governing rights and limitations.
//
// You should have received a copy of the GPL along with this
// program. If not, go to http://www.gnu.org/licenses/gpl.html
// or write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
//
// END SONGBIRD GPL
//
*/


/* This file implements cover scanning. It contains three objects:
 *  sbCoverScanner:
 *    An XPCOM service that activates on startup and on
 *    songbird-library-manager-ready creates a LibraryScanner for the main
 *    library.
 *
 *  LibraryScanner:
 *    A JavaScript object that tries to keep a particular library's cover art
 *    up to date. It listens for library changes and uses an AsyncGUIDArray
 *    to search for and enumerate items for whom cover art should be fetched.
 *    It uses a timer to drive a lot of the work so that it can be throttled.
 *    Currently there's just one instance, created by sbCoverScanner which
 *    scans the main library.
 *
 *  CoverDownloader:
 *    A JavaScript object that handles the downloading of covers from HTTP
 *    URLs to local files. It's used by a LibraryScanner when a fetcher 
 *    returns an HTTP URL.
 *
 * This file works in conjunction with cover fetchers. They are XPCOM
 * services which implement sbICoverFetcher. Currently there are two which
 * are hard-coded - sbFileCoverFetcher and sbAmazonCoverFetcher. Eventually
 * this will be extensible, but not till we've nailed down the functionality
 * and interfaces.
 */

/* TODO:
   Others:
    * We don't rescan when the RESCAN_INTERVAL expires. Should we?
      We will only rescan when something is added to the library.
    * Are we caching failure too aggressively? 
      See LibraryScanner.coverFetchFailed with scope.
    * Move makeDebug() into a jsm.
    * Should sbCoverScanner listen for library addition & removal?
 */

function lookupResult(result) {
  for (var r in Cr) {
    if (Cr[r] == result) return r;
  }
  return result;
}

// return a debug function
function makeDebug(aScope) {
  // return a string representation of a JS object for debugging purposes
  function repr(x) {
    if (x == undefined) {
      return 'undefined';
    } else if (x == null) {
      return 'null';
    } else if (typeof x == 'function') {
      return x.name+'(...)';
    } else if (typeof x == 'string') {
      return x.toSource().match(/^\(new String\((.*)\)\)$/)[1];
    } else if (typeof x == 'number') {
      return x.toSource().match(/^\(new Number\((.*)\)\)$/)[1];
    } else if (typeof x == 'object' && x instanceof Array) {
      var value = '';
      for (var i=0; i<x.length; i++) {
        if (i) value = value + ', ';
        value = value + repr(x[i]);
      }
      return '['+value+']';
    } else if (x instanceof Ci.nsISupports) {
      return x.toString();
    } else {
      return x.toSource();
    }
  }

  // the debug function
  return function DEBUG(aMessage) {
    var msg = aScope+': '+DEBUG.caller.name;
    if (aMessage != undefined) {
      // if there's a message, use it
      msg = msg + ': '+repr(aMessage);
    } else {
      // show the arguments
      msg = msg + '(';
      for (var i=0; i<DEBUG.caller.arguments.length; i++) {
        if (i != 0) { msg = msg + ', '; }
        msg = msg + DEBUG.caller.arguments[i];
      }
      msg = msg + ')';
    }
    dump(msg + '\n');
  }
}

//var DEBUG = makeDebug('sbCoverScanner');
var DEBUG = function(msg) { }

function StringEnumerator(arr) {
  // copy the array
  arr = new Array(arr);
  this.getNext = function() { return arr.pop(0); }
  this.hasMore = function() { return arr.length > 0; }
}

// get unix time
function now() {
  return parseInt((new Date()).getTime()/1000);
}

const Ci = Components.interfaces;
const Cc = Components.classes;
const Cr = Components.results;

Components.utils.import("resource://app/jsmodules/sbProperties.jsm");
Components.utils.import("resource://app/jsmodules/sbLibraryUtils.jsm");

const PROP_LAST_COVER_SCAN = SBProperties.base + 'lastCoverScan';
// rescan every week
const RESCAN_INTERVAL = (60*60*24*7);
// do something every 300ms
const TIMER_INTERVAL = 300;


/* CoverDownloader downloads cover art using Mozilla's nsIWebBrowserPersist and
 * then calls into the LibraryScanner using the sbICoverListener interface. */
function CoverDownloader(aLibraryScanner, aMediaItem, aScope, aURI) {
  DEBUG();
  // we'll need the io service for a couple of things...
  var ioService =  Cc['@mozilla.org/network/io-service;1']
    .getService(Ci.nsIIOService);

  // track shutdown state
  this._shutdown = false;

  // save off the arguments we'll be needing later...
  this._libraryScanner = aLibraryScanner;
  this._mediaItem = aMediaItem;
  this._scope = aScope;

  // default to .jpg
  var extension = '.jpg';
  try {
    // try to extract the extension from download URL
    extension = aURI.match(/\.[a-zA-Z0-9]+$/)[0];
  } catch(e) {
    DEBUG(e);
    DEBUG('failed to extract the extension from :'+aURI);
  }

  // work out where we'll save this
  var albumArtService = Cc['@songbirdnest.com/songbird-album-art-service;1']
    .getService(Ci.sbIAlbumArtService);
  var location = albumArtService.getCoverDownloadLocation(aMediaItem);
  if (location) {
    var file = Cc['@mozilla.org/file/local;1']
      .createInstance(Ci.nsILocalFile);
    try {
      file.initWithPath(location+extension);
    } catch(e) {
      DEBUG(e)
      Components.utils.reportError(e);
      aLibraryScanner.coverFetchFailed(aMediaItem, aScope);
      return;
    }

    this._url = ioService.newFileURI(file).spec;

    this._wbp = Cc['@mozilla.org/embedding/browser/nsWebBrowserPersist;1']
      .createInstance(Ci.nsIWebBrowserPersist);
    this._wbp.persistFlags = Ci.nsIWebBrowserPersist.PERSIST_FLAGS_NONE;
    this._wbp.progressListener = this;
    DEBUG('beginning download');
    this._wbp.saveURI(ioService.newURI(aURI, null, null), null, null, null, 
        null, file);
  } else {
    DEBUG("couldn't determine the cover location");
    aLibraryScanner.coverFetchFailed(aMediaItem, aScope);
  }
}
/* nsISupports */
CoverDownloader.prototype.QueryInterface =
function sbCoverScanner_QueryInterface(aIID) {
  if (!aIID.equals(Ci.nsISupports) &&
      !aIID.equals(Ci.nsIWebProgressListener)) {
    throw Cr.NS_ERROR_NO_INTERFACE;
  }
  return this;
}

/* nsIWebProgressListener methods */
CoverDownloader.prototype.onLocationChange = function (a, b, c) { }
CoverDownloader.prototype.onProgressChange = function (a, b, c, d, e, f) { }
CoverDownloader.prototype.onSecurityChange = function (a, b, c) { }
CoverDownloader.prototype.onStatusChange = function (a, b, c, d) { }
CoverDownloader.prototype.onStateChange =
function CoverDownloader_onStateChange(aWebProgress, aRequest, aStateFlags, 
    aStatus) {
  DEBUG();

  // if we're shutting down, do nothing
  if (this._shutdown) {
    return;
  }

  // when the transfer is complete...
  if (aStateFlags & Ci.nsIWebProgressListener.STATE_STOP) {
    if (aStatus == 0) {
      this._libraryScanner.coverFetchSucceeded(this._mediaItem, this._scope,
         this._url);
    } else {
      this._libraryScanner.coverFetchFailed(this._mediaItem, this._scope);
    }
  }
}

CoverDownloader.prototype.shutdown =
function CoverDownloader_shutdown() {
  DEBUG();
  this._shutdown = true;
}

/* LibraryScanner is the type that looks for tracks in a library that need to
 * have their covers fetched and then asks the album cover service to find
 * them. */
function LibraryScanner(aLibrary) {
  // hang onto this library
  this._library = aLibrary.QueryInterface(Ci.sbILocalDatabaseLibrary);

  // we want to know when new items are added and when a couple of 
  // properties change
  var properties = 
    Cc['@songbirdnest.com/Songbird/Properties/MutablePropertyArray;1']
    .createInstance(Ci.sbIMutablePropertyArray);
  properties.appendProperty(SBProperties.albumName, "");
  properties.appendProperty(SBProperties.artistName, "");
  // FIXME: are there any other properties we need to look for?
  this._library.addListener(this, true, 
      Ci.sbIMediaList.LISTENER_FLAGS_ITEMADDED + 
      Ci.sbIMediaList.LISTENER_FLAGS_ITEMUPDATED + 
      Ci.sbIMediaList.LISTENER_FLAGS_BATCHBEGIN + 
      Ci.sbIMediaList.LISTENER_FLAGS_BATCHEND, properties);

  // our list of sbICoverFetcher implementations
  // FIXME: register these with the category manager
  this._fetchers = [
    Cc['@songbirdnest.com/Songbird/cover/file-fetcher;1']
      .getService(Ci.sbICoverFetcher),
    Cc['@songbirdnest.com/Songbird/cover/amazon-fetcher;1']
      .getService(Ci.sbICoverFetcher),
  ];

  // get an async guid array
  this._array = 
    Cc['@songbirdnest.com/Songbird/Library/LocalDatabase/AsyncGUIDArray;1']
    .createInstance(Ci.sbILocalDatabaseAsyncGUIDArray);
  DEBUG('array: '+this._array);
  this._array.databaseGUID = this._library.databaseGuid;
  this._array.propertyCache = this._library.propertyCache;
  this._array.baseTable = 'media_items';
  // Sort by primaryImageURL
  this._array.addSort(SBProperties.primaryImageURL, true);
  this._array.addSort(PROP_LAST_COVER_SCAN, true);

  // callbacks come to this object
  this._array.addAsyncListener(this);

  // the io service comes in handy
  this._ioService = Cc['@mozilla.org/network/io-service;1']
    .getService(Ci.nsIIOService);

  // are we currently processing anything?
  this._processing = false;

  // are we shutting down?
  this._shutdown = false;

  // after we think we're done, should we try again anyway?
  // this way when we get a notification that the library changed in a way 
  // that might affect cover scanning, we don't interrupt what we're doing.
  // yes, it's named after a song by pulp.
  this._somethingChanged = false;

  // track what's going on during batched operations
  this._batchHelper = new LibraryUtils.BatchHelper();
  this._somethingChangedInBatch = false;

  // set up a timer to do work on 
  this._timer = Cc['@mozilla.org/timer;1'].createInstance(Ci.nsITimer);
  // is the timer running?
  this._timerRunning = false

  // start a scan
  this._startScan();
}

LibraryScanner.prototype.QueryInterface =
function sbCoverScanner_QueryInterface(aIID) {
  if (!aIID.equals(Ci.nsISupports) &&
      !aIID.equals(Ci.nsISupportsWeakReference) &&
      !aIID.equals(Ci.nsITimerCallback) &&
      !aIID.equals(Ci.sbICoverListener) &&
      !aIID.equals(Ci.sbIMediaListListener) &&
      !aIID.equals(Ci.sbILocalDatabaseAsyncGUIDArrayListener)) {
    throw Cr.NS_ERROR_NO_INTERFACE;
  }
  return this;
}

// start a new scan of the library
LibraryScanner.prototype._startScan =
function LibraryScanner_startScan() {
  DEBUG();

  // if we're shutting down, do nothing
  if (this._shutdown) {
    this._stopScan();
    return;
  }

  // invalidate the guid array
  this._array.invalidate();

  // invalidate our counters
  this._length = -1;
  this._index = 0;
  this._foundNothing = true;

  // if the timer isn't running, start it
  if (!this._timerRunning) {
    // every second or so...
    this._timer.initWithCallback(this, TIMER_INTERVAL, 
        Ci.nsITimer.TYPE_REPEATING_SLACK);
    this._timerRunning = true;
  }

  // ask for the length
  this._array.getLengthAsync();
}

// stop the scanning of the library
LibraryScanner.prototype._stopScan =
function LibraryScanner_stopScan() {
  DEBUG();

  if (this._somethingChanged && !this._shutdown) {
    // we think we should stop, but something has changed.
    // lets start another scan
    DEBUG('something changed - lets start again');
    this._somethingChanged = false;
    this._startScan();
    return;
  }

  if (this._timerRunning) {
    this._timer.cancel();
    this._timerRunning = false;
  }

}

// sbILocalDatabaseAsyncGUIDArrayListener
LibraryScanner.prototype.onGetLength = 
function LibraryScanner_onGetLength(aLength, aResult) {
  this._length = aLength;
}

// nsITimerCallback
LibraryScanner.prototype.notify = 
function LibraryScanner_notify(aTimer) {
  DEBUG('processing='+this._processing+' length='+this._length+' index='+this._index);
  DEBUG(' processing='+this._processing+' timerRunning='+this._timerRunning);
  DEBUG(' something_changed='+this._somethingChanged+' found_nothing='+this._foundNothing);
  DEBUG(' batch_helper.depth='+this._batchHelper.depth()+' something_changed_in_batch='+this._somethingChangedInBatch);

  // if we're shutting down, do nothing
  if (this._shutdown) {
    this._stopScan();
    return;
  }

  if (this._batchHelper.isActive()) {
    // don't do anything during a batch operation
    return;
  }

  if (this._processing) {
    // if we're already processing, do nothing
    return;
  }

  if (this._length == -1) {
    // we're waiting for the length of the result set
    // so do nothing right now
    return;
  }

  // if there are no items to scan, then stop
  if (this._length == 0) {
    DEBUG('there are no items to scan');
    this._stopScan();
    return;
  }

  // if we have run out of items then stop
  if (this._index >= this._length) {
    DEBUG('ran out of items to scan');
    this._stopScan();
    return;
  }

  // we know the length. 
  // find out the guid of the next item
  this._array.getGuidByIndexAsync(this._index++);
}

// sbILocalDatabaseAsyncGUIDArrayListener
LibraryScanner.prototype.onGetGuidByIndex = 
function LibraryScanner_onGetGuidByIndex(aIndex, aGuid, aResult) {
  DEBUG();

  // if we're shutting down, do nothing
  if (this._shutdown) {
    this._stopScan();
    return;
  }

  if (aGuid == 'error') {
    /* I think this is okay */
    this._startScan();
    return;
  }

  // get the media item for the guid
  var item = this._library.getMediaItem(aGuid);

  var artist = item.getProperty(SBProperties.albumName);
  var album = item.getProperty(SBProperties.artistName);
  // do we have album art?
  DEBUG('artist/album: '+artist+'/'+album);
  var art = item.getProperty(SBProperties.primaryImageURL);
  DEBUG('art? '+art);
  var lastCoverScan = parseInt(item.getProperty(PROP_LAST_COVER_SCAN));
  if (isNaN(lastCoverScan)) { lastCoverScan = 0; }
  DEBUG('last scan: '+lastCoverScan);

  /* Due to our ordering, items should be sorted by primaryImageURL and 
   * then by lastCoverScan. This means when we see an item with art or
   * an item which was scanned more recently than RESCAN_INTERVAL we can
   * stop.
   */
  if (!art) {
    if (lastCoverScan+RESCAN_INTERVAL < now()) {
      if (artist && album) {
        this._processing = true
        this._fetchCover(item);
      }
    } else {
      DEBUG('the most recent scanable item has been scanned to recently');
      this._stopScan();
    }
  } else {
    // if we get to an item which already has art then we've processed everything
    DEBUG('finished scanning all the items without art');
    this._stopScan();
  }
}

// begin fetching cover art
LibraryScanner.prototype._fetchCover =
function LibraryScanner_fetchCover(aMediaItem) {
  DEBUG();

  this._currentMediaItem = aMediaItem;
  this._fetcherIndex = 0;
  this._askNextFetcher();
}

// ask the next cover art fetcher to find art
LibraryScanner.prototype._askNextFetcher =
function LibraryScanner_askNextFetcher() {
  DEBUG();

  // if we're shutting down, do nothing
  if (this._shutdown) {
    this._stopScan();
    return;
  }

  if (this._fetcherIndex >= this._fetchers.length) {
    // we ran out of fetchers!
    // EPIC FAIL
    // FIXME: mark this item as unfetchable somehow
    this._processing = false;
    return;
  }

  // ask the next fetcher to find our cover art
  this._fetchers[this._fetcherIndex++]
    .fetchCoverForMediaItem(this._currentMediaItem, this);
}

// sbICoverListener
LibraryScanner.prototype.coverFetchSucceeded =
function LibraryScanner_coverFetchSucceeded(aMediaItem, aScope, aURI) {
  DEBUG('aMediaItem='+aMediaItem.guid+' aURI='+aURI);

  // if we're shutting down, do nothing
  if (this._shutdown) {
    this._stopScan();
    return;
  }

  // if aURL isn't file:///, we need to download it...
  var uri = this._ioService.newURI(aURI, null, null);
  if (uri.scheme != 'file') {
    DEBUG('starting cover downloader');
    this._downloader = new CoverDownloader(this, aMediaItem, aScope, aURI);
  } else {
    // we found something
    this._foundNothing = false;
    if (aScope) {
      // get a view representing the scope
      var view = this._library.createView();

      // add filters from the scope
      for (var i=0; i<aScope.length; i++) {
        property = aScope.getPropertyAt(i);
        DEBUG('appending property '+property.id+' = '+property.value);
        view.cascadeFilterSet.appendFilter(property.id);
        view.cascadeFilterSet.set(i, [property.value], 1);
      }

      DEBUG('view.length = '+view.length);

      this._library.runInBatchMode(function() {
        var rightNow = now();
        for (var i=0; i<view.length; i++) {
          // FIXME: should we override the existing value?
          try {
            var item = view.getItemByIndex(i);
            item.setProperty(SBProperties.primaryImageURL, aURI);
            item.setProperty(PROP_LAST_COVER_SCAN, rightNow);
          } catch(e) {
            DEBUG(e);
            Components.utils.reportError(e);
          }
        }
      });

      // restart this thing
      this._processing = false;
      this._startScan();
    } else {
      // set the cover art on a single media item
      try {
        aMediaItem.setProperty(SBProperties.primaryImageURL, aURI);
        var rightNow = now();
        DEBUG('setting property to: '+rightNow);
        aMediaItem.setProperty(PROP_LAST_COVER_SCAN, rightNow);
      } catch(e) { 
        DEBUG(e);
        Components.utils.reportError(e);
      }
      this._processing = false;
    }

  }
}

// sbICoverListener
LibraryScanner.prototype.coverFetchFailed =
function LibraryScanner_coverFetchFailed(aMediaItem, aScope) {
  DEBUG();

  // if we're shutting down, do nothing
  if (this._shutdown) {
    this._stopScan();
    return;
  }

  // update the lastCoverScan property
  var rightNow = now();
  if (aScope) {
    // get a view representing the scope
    var view = this._library.createView();

    // add filters from the scope
    for (var i=0; i<aScope.length; i++) {
      property = aScope.getPropertyAt(i);
      view.cascadeFilterSet.appendFilter(property.id);
      view.cascadeFilterSet.set(i, [property.value], 1);
    }

    this._library.runInBatchMode(function() {
      var rightNow = now();
      for (var i=0; i<view.length; i++) {
        // FIXME: should we override the existing value?
        var item = view.getItemByIndex(i);
        item.setProperty(PROP_LAST_COVER_SCAN, rightNow);
      }
    });
  } else {
    aMediaItem.setProperty(PROP_LAST_COVER_SCAN, rightNow);
  }

  // oh, we failed?
  // let's try the next fetcher
  this._askNextFetcher();
}

// the library changed
LibraryScanner.prototype._libraryChanged =
function LibraryScanner_libraryChanged() {
  if (this._batchHelper.isActive()) {
    // there is batch processing happening, let's not do anything right now
    this._somethingChangedInBatch = true;
  } else {
    if (this._timerRunning) {
      // something is going on, let's not interrupt it
      this._somethingChanged = true;
    } else {
      // nothing is going on, let's start a new scan
      this._startScan();
    }
  }
}

// sbIMediaListListener
LibraryScanner.prototype.onItemAdded =
function LibraryScanner_onItemAdded(aMediaList, aMediaItem, aIndex) {
  this._libraryChanged();
  return true;
}

// sbIMediaListListener
LibraryScanner.prototype.onItemUpdated =
function LibraryScanner_onItemUpdated(aMediaList, aMediaItem, aIndex) {
  this._libraryChanged();
  return true
}

// sbIMediaListListener
LibraryScanner.prototype.onBatchBegin =
function LibraryScanner_onBatchBegin(aMediaList) {
  this._batchHelper.begin();
}

// sbIMediaListListener
LibraryScanner.prototype.onBatchEnd =
function LibraryScanner_onBatchEnd(aMediaList) {
  this._batchHelper.end();
  if (!this._batchHelper.isActive()) {
    if (this._somethingChangedInBatch) {
      this._libraryChanged();
    }
    this._somethingChangedInBatch = false;
  }
}

LibraryScanner.prototype.shutdown =
function LibraryScanner_shutdown() {
  DEBUG();

  // we're shutting down
  this._shutdown = true;
  
  // shutdown our downloader
  if (this._downloader) {
    this._downloader.shutdown();
  }

  // shutdown our fetchers
  for (var i=0; i<this._fetchers.length; i++) {
    this._fetchers[i].shutdown();
  }
}


function sbCoverScanner() {
  DEBUG();
  // track our LibraryScanner objects
  // it's a guid->LibraryScanner() map
  this._libraryScanners = {}

  // the observer service is handy to have
  this._observerSvc = Cc['@mozilla.org/observer-service;1']
    .getService(Ci.nsIObserverService);

  // we want to know when the library manager is ready
  this._observerSvc.addObserver(this, 'songbird-library-manager-ready', false);
  // and when it's going away
  this._observerSvc.addObserver(this, 'profile-change-teardown', false);
}
sbCoverScanner.prototype = {
  classDescription: "Songbird Album Cover Scanner",
  classID: Components.ID("{62a41471-3700-4fa6-b0da-ab045b433231}"),
  contractID: "@songbirdnest.com/Songbird/CoverScanner;1"
}

// nsISupports
sbCoverScanner.prototype.QueryInterface =
function sbCoverScanner_QueryInterface(aIID) {
  if(!aIID.equals(Ci.nsISupports) &&
     !aIID.equals(Ci.nsIObserver)) {
    throw Cr.NS_ERROR_NO_INTERFACE;
  }
  return this;
}

// nsIObserver
sbCoverScanner.prototype.observe = 
function sbCoverScanner_observe(aSubject, aTopic, aData) {
  // dispatch observers by topic
  if (this[aTopic]) {
    this[aTopic](aSubject, aTopic, aData);
  }
}


sbCoverScanner.prototype['songbird-library-manager-ready'] =
function sbCoverScanner_songbird_library_manager_ready(aSubject, aTopic, aData){
  DEBUG();

  // add a property indicating how long it's been since we've last scanned
  var manager = Cc['@songbirdnest.com/Songbird/Properties/PropertyManager;1']
    .getService(Ci.sbIPropertyManager);
  var property = Cc['@songbirdnest.com/Songbird/Properties/Info/Datetime;1']
    .createInstance(Ci.sbIDatetimePropertyInfo);
  property.nullSort = Ci.sbIPropertyInfo.SORT_NULL_SMALL;
  property.id = PROP_LAST_COVER_SCAN;
  property.displayName = 'Last Cover Scan';
  // this property is internal
  property.userViewable = false;
  property.userEditable = false;
  property.remoteReadable = false;
  property.remoteWritable = false;
  property.timeType = Ci.sbIDatetimePropertyInfo.TIMETYPE_TIMESTAMP;
  /*
  property.minTime = 0;
  property.maxTime = 0;
  */
  manager.addPropertyInfo(property);
  DEBUG('added property: '+property);

  // create a cover scanner for the main library
  DEBUG('main library guid = '+LibraryUtils.mainLibrary.guid);
  this._libraryScanners[LibraryUtils.mainLibrary.guid] = 
      new LibraryScanner(LibraryUtils.mainLibrary);
}

sbCoverScanner.prototype['profile-change-teardown'] =
function sbCoverScanner_profile_change_teardown() {
  DEBUG();
  for (var guid in this._libraryScanners) {
    this._libraryScanners[guid].shutdown();
  }
}




Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");

function NSGetModule(compMgr, fileSpec) {
  return XPCOMUtils.generateModule([
    sbCoverScanner,
  ],
  function(aCompMgr, aFileSpec, aLocation) {
    XPCOMUtils.categoryManager.addCategoryEntry(
      "app-startup",
      sbCoverScanner.prototype.classDescription,
      "service," + sbCoverScanner.prototype.contractID,
      true,
      true);
  });
}


