/*
//
// Copyright(c) 2005-2008 POTI, Inc.
// http://songbirdnest.com
//
// For information about the licensing and copyright of this Add-On please
// contact POTI, Inc. at customer@songbirdnest.com.
//
*/

/* *****************************************************************************
 *******************************************************************************
 *
 * iPod device request UI handler component.
 *
 *   These services handle iPod device requests by presenting the user with
 * dialog boxes.
 *
 *******************************************************************************
 ******************************************************************************/

/** 
* \file  IPodDeviceRequestUIHandler.js
* \brief Javascript source for the iPod device request UI handler component.
*/

/* *****************************************************************************
 *
 * iPod device request UI handler component defs.
 *
 ******************************************************************************/

if (typeof(Cc) == "undefined")
  var Cc = Components.classes;
if (typeof(Ci) == "undefined")
  var Ci = Components.interfaces;
if (typeof(Cr) == "undefined")
  var Cr = Components.results;


/* *****************************************************************************
 *
 * iPod device request UI handler component configuration.
 *
 ******************************************************************************/

/*
 * className                    Name of component class.
 * cid                          Component CID.
 * contractID                   Component contract ID.
 * ifList                       List of external component interfaces.
 * categoryList                 List of component categories.
 *
 * deviceCategoryName           Name of device category.
 * localeBundlePath             Path to locale string bundle.
 */

var IPD_RUHCfg = {};

IPD_RUHCfg.className = "Songbird iPod Device Request UI Handler";
IPD_RUHCfg.cid = Components.ID("{e9c88748-8e34-4c17-8d37-1a41437c5a5b}");
IPD_RUHCfg.contractID =
            "@songbirdnest.com/Songbird/Device/IPodDeviceRequestUIHandler;1";
IPD_RUHCfg.ifList = [ Components.interfaces.sbIIPodDeviceRequestUIHandler,
                      Components.interfaces.sbIIPodDeviceRequestHandler,
                      Components.interfaces.nsISupports ];
IPD_RUHCfg.categoryList = [];

IPD_RUHCfg.deviceCategoryName = "Apple iPod";
IPD_RUHCfg.localeBundlePath = "chrome://ipod/locale/IPodDevice.properties";


/* *****************************************************************************
 *
 * iPod device request UI handler services.
 *
 ******************************************************************************/

/*
 * IPD_RUH
 *
 *   This function constructs an iPod device request UI handler object.
 */

function IPD_RUH()
{
}

/* Set the constructor. */
IPD_RUH.prototype.constructor = IPD_RUH;


/*
 * iPod device request UI handler services fields.
 *
 *   mCfg                       Configuration settings.
 *   mLocale                    Locale string bundle.
 *   mDevMgr                    Device manager sbIDeviceManager object.
 *   mDevCatMgr                 Device category manager sbIIPodDevice object.
 *   mParentWindow              Parent window to use for UI.
 */

IPD_RUH.prototype.mCfg = IPD_RUHCfg;
IPD_RUH.prototype.mLocale = null;
IPD_RUH.prototype.mDevMgr = null;
IPD_RUH.prototype.mDevCatMgr = null;
IPD_RUH.prototype.mParentWindow = null;


/* *****************************************************************************
 *
 * iPod device request UI handler sbIIPodDeviceRequestUIHandler services.
 *
 ******************************************************************************/

/**
 * \brief Initialize the iPod device request UI handler.
 *
 * \param aParentWindow The parent window to use with the UI.  May be null.
 */

IPD_RUH.prototype.initialize = function(aParentWindow)
{
    /* Set the parent window. */
    this.mParentWindow = aParentWindow;

    /* Get the device manager. */
    this.mDevMgr = Components.
                        classes["@songbirdnest.com/Songbird/DeviceManager;1"].
                        getService(Components.interfaces.sbIDeviceManager);

    /* Do nothing if device category is not available. */
    if (!this.mDevMgr.hasDeviceForCategory(this.mCfg.deviceCategoryName))
        return;

    /* Get the iPod device utilities. */
    var jsLoader = Components.classes["@mozilla.org/moz/jssubscript-loader;1"]
                             .getService
                                (Components.interfaces.mozIJSSubScriptLoader);
    jsLoader.loadSubScript("chrome://ipod/content/scripts/IPodDeviceUtils.js",
                           null);

    /* Get the locale string bundle. */
    {
        var                         stringBundleService;

        stringBundleService =
                Components.classes["@mozilla.org/intl/stringbundle;1"].
                    getService(Components.interfaces.nsIStringBundleService);
        this.mLocale = stringBundleService.createBundle
                                                (this.mCfg.localeBundlePath);
    }

    /* Initialize the window services. */
    IPDWindow.initialize(this.mLocale);

    /* Get the device category manager. */
    this.mDevCatMgr = this.mDevMgr.getDeviceByCategory
                                                (this.mCfg.deviceCategoryName);
    this.mDevCatMgr = this.mDevCatMgr
                          .QueryInterface(Components.interfaces.sbIIPodDevice);
}


/**
 * \brief Finalize the iPod device request UI handler.
 */

IPD_RUH.prototype.finalize = function()
{
    /* Finalize the iPod device utilities. */
    IPDUtils.finalize();

    /* Remove some object references. */
    this.mLocale = null;
    this.mDevMgr = null;
    this.mDevCatMgr = null;
    this.mParentWindow = null;
}


/* *****************************************************************************
 *
 * iPod device request UI handler sbIIPodDeviceRequestHandler services.
 *
 ******************************************************************************/

/**
 * \brief Handle a request to change the library that the device is linked
 * to for syncing.
 *
 * \param aDeviceIdentifier The device unique identifier.
 *
 * \return True if request is granted.
 */

IPD_RUH.prototype.changeLink = function(aDeviceIdentifier)
{
    var                         readableName;
    var                         response = {};
    var                         granted;

    /* Get the device readable name. */
    readableName = this.mDevCatMgr.getLibrary(aDeviceIdentifier).name;

    /* Query the user for the action to take. */
    IPDWindow.openDialog(this.mParentWindow,
                         "chrome://ipod/content/xul/SimpleDialog.xul",
                         [ "confirm_dialog.change_link_lib.title",
                           [ "confirm_dialog.change_link_lib.msg",
                             readableName ],
                           "window.yes",
                           "window.no" ],
                         [ response ]);
    response = response.value;

    /* Determine whether the request is granted. */
    if (response == "positive")
        granted = true;
    else
        granted = false;

    return (granted);
}


/**
 * \brief Handle a request to create a sync playlist due to an overcapacity
 * condition.
 *
 * \param aDeviceIdentifier The device unique identifier.
 *
 * \return True if request is granted.
 */

IPD_RUH.prototype.createSyncPlaylist = function(aDeviceIdentifier)
{
    var                         readableName;
    var                         response = {};
    var                         granted;

    /* Get the device readable name. */
    readableName = this.mDevCatMgr.getLibrary(aDeviceIdentifier).name;

    /* Query the user for the action to take. */
    IPDWindow.openDialog(this.mParentWindow,
                         "chrome://ipod/content/xul/SimpleDialog.xul",
                         [ "confirm_dialog.sync.capacity_exceeded.title",
                           [ "confirm_dialog.sync.capacity_exceeded.msg",
                             readableName ],
                           "window.ok",
                           "window.cancel" ],
                         [ response ]);
    response = response.value;

    /* Determine whether the request is granted. */
    if (response == "positive")
        granted = true;
    else
        granted = false;

    return (granted);
}


/**
 * \brief Handle a request to get the action to take in response to an
 *        application quit event occuring while a device is busy.
 *
 * \return Action value.
 */

IPD_RUH.prototype.getQuitWhileBusyAction = function()
{
    var                         response = {};
    var                         waitForCompletion;

    /* Query the user for the action to take. */
    IPDWindow.openDialog(this.mParentWindow,
                         "chrome://ipod/content/xul/SimpleDialog.xul",
                         [ "confirm_dialog.shutdown_interrupt.title",
                           "confirm_dialog.shutdown_interrupt.msg",
                           "window.yes",
                           "window.no" ],
                         [ response ]);
    response = response.value;

    /* Determine whether to wait for completion of iPod operations. */
    if (response == "positive")
    {
        waitForCompletion = Components.interfaces.sbIIPodDeviceRequestHandler
                                                 .WAIT_FOR_COMPLETION;
    }
    else
    {
        waitForCompletion = Components.interfaces.sbIIPodDeviceRequestHandler
                                                 .DONT_WAIT_FOR_COMPLETION;
    }

    return (waitForCompletion);
}


/**
 * \brief Handle a request to wait until all devices have completed all
 *        operations.  The handler should notify the user that the
 *        application shutdown is waiting for devices to complete
 *        operations.
 */

IPD_RUH.prototype.waitForCompletion = function()
{
    IPDWindow.openDialog(this.mParentWindow,
                         "chrome://ipod/content/xul/WaitForCompletion.xul",
                         [],
                         []);
}


/**
 * \brief Handle a request to warn the user that the media items specified
 *        by aUnsupportedMediaItems are not supported by the device
 *        specified by aDeviceIdentifier.
 *
 * \param aDeviceIdentifier         The device unique identifier.
 * \param aUnsupportedMediaCount    Number of items in
 *                                  aUnsupportedMediaItems.
 * \param aUnsupportedMediaItems    List of unsupported media items.
 */

IPD_RUH.prototype.unsupportedMediaItems = function(aDeviceIdentifier,
                                                   aUnsupportedMediaItemCount,
                                                   aUnsupportedMediaItems)
{
    try { this._unsupportedMediaItems(aDeviceIdentifier,
                                      aUnsupportedMediaItemCount,
                                      aUnsupportedMediaItems); }
    catch (ex) { IPDLogException(ex); }
}

IPD_RUH.prototype._unsupportedMediaItems = function(aDeviceIdentifier,
                                                    aUnsupportedMediaItemCount,
                                                    aUnsupportedMediaItems)
{
    /* Convert the array of media items to an nsIArray. */
    var unsupportedMediaItems = Cc["@mozilla.org/array;1"]
                                    .createInstance(Ci.nsIMutableArray);
    for (var i = 0; i < aUnsupportedMediaItems.length; i++)
    {
        unsupportedMediaItems.appendElement(aUnsupportedMediaItems[i], false);
    }
 
    /* Warn the user. */
    IPDWindow.openDialog(this.mParentWindow,
                         "chrome://ipod/content/xul/unsupportedMedia.xul",
                         [ aDeviceIdentifier, unsupportedMediaItems ],
                         []);
}


/**
 * \brief Handle a request to warn the user that the device specified by
 *        aDeviceIdentifier was formatted with an unsupported file system.
 *
 * \param aDeviceIdentifier         The device unique identifier.
 */

IPD_RUH.prototype.unsupportedFileSystem = function(aDeviceIdentifier)
{
    /* Warn the user. */
    IPDWindow.openDialog(this.mParentWindow,
                         "chrome://ipod/content/xul/SimpleDialog.xul",
                         [ "ipod.dialog.unsupported_filesystem.title",
                           "ipod.dialog.unsupported_filesystem.msg",
                           "window.ok" ],
                         []);
}


/* *****************************************************************************
 *
 * iPod device request UI handler nsISupports services.
 *
 ******************************************************************************/

/*
 * QueryInterface
 *
 *   --> aIID                   Requested interface.
 *
 *   NS_ERROR_NO_INTERFACE      Requested interface is not supported.
 *
 *   This function returns a component object implementing the interface
 * specified by aIID.
 */

IPD_RUH.prototype.QueryInterface = function(aIID)
{
    var                         interfaceSupported = false;
    var                         i;

    /* Check for supported interfaces. */
    for (i = 0; i < this.mCfg.ifList.length; i++)
    {
        if (aIID.equals(this.mCfg.ifList[i]))
        {
            interfaceSupported = true;
            break;
        }
    }
    if (!interfaceSupported)
        throw(Components.results.NS_ERROR_NO_INTERFACE);

    return(this);
}


/* *****************************************************************************
 *******************************************************************************
 *
 * iPod device request UI handler component module services.
 *
 *******************************************************************************
 ******************************************************************************/

/*
 * Component module object.
 *
 *   mCfg                       Configuration settings.
 */

var ComponentModule = {};
ComponentModule.mCfg = IPD_RUHCfg;


/*
 * NSGetModule
 *
 *   This function returns the component module.
 */

function NSGetModule(comMgr, fileSpec)
{
    return (ComponentModule);
}


/*
 * registerSelf
 *
 *   This function registers the component module.
 */

ComponentModule.registerSelf = function(compMgr, fileSpec, location, type)
{
    var                         categoryManager;
    var                         category;
    var                         i;

    /* Register the component factory. */
    compReg = compMgr.QueryInterface
                                (Components.interfaces.nsIComponentRegistrar);
    compReg.registerFactoryLocation(this.mCfg.cid,
                                    this.mCfg.className,
                                    this.mCfg.contractID,
                                    fileSpec,
                                    location,
                                    type);

    /* Register any categories. */
    if (this.mCfg.categoryList && this.mCfg.categoryList.length)
    {
        categoryManager = Components
                        .classes["@mozilla.org/categorymanager;1"]
                        .getService(Components.interfaces.nsICategoryManager);
        for (i = 0; i < this.mCfg.categoryList.length; i++)
        {
            category = this.mCfg.categoryList[i];
            categoryManager.addCategoryEntry(category.category,
                                             category.entry,
                                             category.value,
                                             true,
                                             true);
        }
    }
}


/*
 * unregisterSelf
 *
 *   This function unregisters the component module.
 */

ComponentModule.unregisterSelf = function(compMgr, location, type)
{
    var                         categoryManager;

    /* Unregister the component factory. */
    compReg = compMgr.QueryInterface
                                (Components.interfaces.nsIComponentRegistrar);
    compReg.unregisterFactoryLocation(this.mCfg.cid, location);

    /* Unregister any categories. */
    if (this.mCfg.categoryList && this.mCfg.categoryList.length)
    {
        categoryManager = Components
                        .classes["@mozilla.org/categorymanager;1"]
                        .getService(Components.interfaces.nsICategoryManager);
        for (i = 0; i < this.mCfg.categoryList.length; i++)
        {
            category = this.mCfg.categoryList[i];
            categoryManager.deleteCategoryEntry(category.category,
                                                category.entry,
                                                true);
        }
    }
}


/*
 * getClassObject
 *
 *   This function returns a factory for the component.
 */

ComponentModule.getClassObject = function(compMgr, classID, interfaceID)
{
    /* Check for supported interfaces. */
    if (!interfaceID.equals(Components.interfaces.nsIFactory))
        throw(Components.results.NS_ERROR_NOT_IMPLEMENTED);

    /* Check for supported component types. */
    if (!classID.equals(this.mCfg.cid))
        throw(Components.results.NS_ERROR_NO_INTERFACE);

    return(ComponentFactory);
}


/*
 * canUnload
 *
 *   Not sure what this does.
 */

ComponentModule.canUnload = function(compMgr)
{
    return true;
}


/* *****************************************************************************
 *******************************************************************************
 *
 * iPod device service pane module component factory.
 *
 *******************************************************************************
 ******************************************************************************/

/*
 * Component factory object.
 *
 *   mCfg                       Configuration settings.
 */

var ComponentFactory = {};
ComponentFactory.mCfg = IPD_RUHCfg;


/*
 * createInstance
 *
 *   --> outer                  ???
 *   --> interfaceID            ID of interface instance to create.
 *
 *   This function creates an instance of the component providing the
 * interface specified by interfaceID.
 */

ComponentFactory.createInstance = function(outer, interfaceID)
{
    var                         instance;
    var                         interfaceSupported = false;
    var                         i;

    /*zzz?*/
    if (outer != null)
        throw (Components.results.NS_ERROR_NO_AGGREGATION);

    /* Check for supported interfaces. */
    for (i = 0; i < this.mCfg.ifList.length; i++)
    {
        if (interfaceID.equals(this.mCfg.ifList[i]))
        {
            interfaceSupported = true;
            break;
        }
    }
    if (!interfaceSupported)
        throw Components.results.NS_ERROR_INVALID_ARG;

    /* Create an instance of the component. */
    instance = (new IPD_RUH()).QueryInterface(interfaceID);

    return (instance);
}


